# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data_string import AdditionalDataString
from .annualrevenue import Annualrevenue
from .app_id import AppId
from .application_details_record_messages_item import ApplicationDetailsRecordMessagesItem
from .avgmonthly import Avgmonthly
from .baddress_1 import Baddress1
from .baddress_2 import Baddress2
from .bank_data import BankData
from .bcity import Bcity
from .bcountry import Bcountry
from .binperson import Binperson
from .binphone import Binphone
from .binweb import Binweb
from .boarding_application_attachments import BoardingApplicationAttachments
from .boarding_link_id import BoardingLinkId
from .boarding_status import BoardingStatus
from .bphone import Bphone
from .bstate import Bstate
from .bsummary import Bsummary
from .builder_data import BuilderData
from .busstartdate import Busstartdate
from .bzip import Bzip
from .contacts_field import ContactsField
from .created_at import CreatedAt
from .dbaname import Dbaname
from .ein import Ein
from .external_paypoint_id import ExternalPaypointId
from .file_content import FileContent
from .general_events import GeneralEvents
from .highticketamt import Highticketamt
from .last_modified import LastModified
from .legalname import Legalname
from .license import License
from .licensestate import Licensestate
from .maddress import Maddress
from .maddress_1 import Maddress1
from .mcountry import Mcountry
from .mstate import Mstate
from .mzip import Mzip
from .org_parent_name import OrgParentName
from .orgid import Orgid
from .own_type import OwnType
from .ownership import Ownership
from .page_identifier import PageIdentifier
from .recipient_email_notification import RecipientEmailNotification
from .resumable import Resumable
from .sales_code import SalesCode
from .services import Services
from .signer_data import SignerData
from .taxfillname import Taxfillname
from .template_id import TemplateId
from .ticketamt import Ticketamt
from .website import Website
from .whencharged import Whencharged
from .whendelivered import Whendelivered
from .whenprovided import Whenprovided
from .whenrefunded import Whenrefunded


class ApplicationDetailsRecord(UniversalBaseModel):
    annual_revenue: typing_extensions.Annotated[
        typing.Optional[Annualrevenue], FieldMetadata(alias="annualRevenue")
    ] = None
    average_monthly_volume: typing_extensions.Annotated[
        typing.Optional[Avgmonthly], FieldMetadata(alias="averageMonthlyVolume")
    ] = None
    average_ticket_amount: typing_extensions.Annotated[
        typing.Optional[Ticketamt], FieldMetadata(alias="averageTicketAmount")
    ] = None
    b_address_1: typing_extensions.Annotated[typing.Optional[Baddress1], FieldMetadata(alias="bAddress1")] = None
    b_address_2: typing_extensions.Annotated[typing.Optional[Baddress2], FieldMetadata(alias="bAddress2")] = None
    bank_data: typing_extensions.Annotated[typing.Optional[BankData], FieldMetadata(alias="bankData")] = None
    b_city: typing_extensions.Annotated[typing.Optional[Bcity], FieldMetadata(alias="bCity")] = None
    b_country: typing_extensions.Annotated[typing.Optional[Bcountry], FieldMetadata(alias="bCountry")] = None
    b_fax: typing_extensions.Annotated[typing.Optional[Bphone], FieldMetadata(alias="bFax")] = pydantic.Field(
        default=None
    )
    """
    The business's fax number.
    """

    bin_person: typing_extensions.Annotated[typing.Optional[Binperson], FieldMetadata(alias="binPerson")] = None
    bin_phone: typing_extensions.Annotated[typing.Optional[Binphone], FieldMetadata(alias="binPhone")] = None
    bin_web: typing_extensions.Annotated[typing.Optional[Binweb], FieldMetadata(alias="binWeb")] = None
    boarding_link_id: typing_extensions.Annotated[
        typing.Optional[BoardingLinkId], FieldMetadata(alias="boardingLinkId")
    ] = None
    boarding_status: typing_extensions.Annotated[
        typing.Optional[BoardingStatus], FieldMetadata(alias="boardingStatus")
    ] = None
    boarding_sub_status: typing_extensions.Annotated[
        typing.Optional[BoardingStatus], FieldMetadata(alias="boardingSubStatus")
    ] = None
    b_phone: typing_extensions.Annotated[typing.Optional[Bphone], FieldMetadata(alias="bPhone")] = None
    b_startdate: typing_extensions.Annotated[typing.Optional[Busstartdate], FieldMetadata(alias="bStartdate")] = None
    b_state: typing_extensions.Annotated[typing.Optional[Bstate], FieldMetadata(alias="bState")] = None
    b_summary: typing_extensions.Annotated[typing.Optional[Bsummary], FieldMetadata(alias="bSummary")] = None
    builder_data: typing_extensions.Annotated[typing.Optional[BuilderData], FieldMetadata(alias="builderData")] = None
    b_zip: typing_extensions.Annotated[typing.Optional[Bzip], FieldMetadata(alias="bZip")] = None
    contact_data: typing_extensions.Annotated[typing.Optional[ContactsField], FieldMetadata(alias="contactData")] = None
    created_at: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="createdAt")] = None
    dba_name: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="dbaName")] = None
    documents_ref: typing_extensions.Annotated[
        typing.Optional[BoardingApplicationAttachments], FieldMetadata(alias="documentsRef")
    ] = None
    ein: typing.Optional[Ein] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointId")
    ] = None
    general_events: typing_extensions.Annotated[
        typing.Optional[typing.List[GeneralEvents]], FieldMetadata(alias="generalEvents")
    ] = pydantic.Field(default=None)
    """
    Events associated with the application.
    """

    high_ticket_amount: typing_extensions.Annotated[
        typing.Optional[Highticketamt], FieldMetadata(alias="highTicketAmount")
    ] = None
    id_application: typing_extensions.Annotated[typing.Optional[AppId], FieldMetadata(alias="idApplication")] = None
    last_modified: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="lastModified")] = (
        None
    )
    legal_name: typing_extensions.Annotated[typing.Optional[Legalname], FieldMetadata(alias="legalName")] = None
    license: typing.Optional[License] = None
    license_state: typing_extensions.Annotated[typing.Optional[Licensestate], FieldMetadata(alias="licenseState")] = (
        None
    )
    logo: typing.Optional[FileContent] = pydantic.Field(default=None)
    """
    Object containing logo file.
    """

    m_address_1: typing_extensions.Annotated[typing.Optional[Maddress], FieldMetadata(alias="mAddress1")] = None
    m_address_2: typing_extensions.Annotated[typing.Optional[Maddress1], FieldMetadata(alias="mAddress2")] = None
    mccid: typing.Optional[str] = None
    m_city: typing_extensions.Annotated[typing.Optional[Mstate], FieldMetadata(alias="mCity")] = None
    m_country: typing_extensions.Annotated[typing.Optional[Mcountry], FieldMetadata(alias="mCountry")] = None
    messages: typing.Optional[typing.List[ApplicationDetailsRecordMessagesItem]] = None
    m_state: typing_extensions.Annotated[typing.Optional[Mstate], FieldMetadata(alias="mState")] = None
    m_zip: typing_extensions.Annotated[typing.Optional[Mzip], FieldMetadata(alias="mZip")] = None
    org_id: typing_extensions.Annotated[typing.Optional[Orgid], FieldMetadata(alias="orgId")] = None
    org_parent_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="orgParentName")
    ] = None
    owner_data: typing_extensions.Annotated[typing.Optional[Ownership], FieldMetadata(alias="ownerData")] = None
    own_type: typing_extensions.Annotated[typing.Optional[OwnType], FieldMetadata(alias="ownType")] = None
    pageidentifier: typing.Optional[PageIdentifier] = None
    recipient_email_notification: typing_extensions.Annotated[
        typing.Optional[RecipientEmailNotification], FieldMetadata(alias="recipientEmailNotification")
    ] = None
    resumable: typing.Optional[Resumable] = None
    sales_code: typing_extensions.Annotated[typing.Optional[SalesCode], FieldMetadata(alias="salesCode")] = None
    service_data: typing_extensions.Annotated[typing.Optional[Services], FieldMetadata(alias="serviceData")] = None
    signer: typing.Optional[SignerData] = None
    taxfillname: typing.Optional[Taxfillname] = None
    template_id: typing_extensions.Annotated[typing.Optional[TemplateId], FieldMetadata(alias="templateId")] = None
    website_address: typing_extensions.Annotated[typing.Optional[Website], FieldMetadata(alias="websiteAddress")] = None
    whencharged: typing.Optional[Whencharged] = None
    whendelivered: typing.Optional[Whendelivered] = None
    when_provided: typing_extensions.Annotated[typing.Optional[Whenprovided], FieldMetadata(alias="whenProvided")] = (
        None
    )
    whenrefund: typing.Optional[Whenrefunded] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString], FieldMetadata(alias="additionalData")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .page_identifier import PageIdentifier
from .pagesize import Pagesize
from .totalpages import Totalpages
from .totalrecords import Totalrecords


class VCardSummary(UniversalBaseModel):
    total_pages: typing_extensions.Annotated[Totalpages, FieldMetadata(alias="totalPages")]
    total_records: typing_extensions.Annotated[Totalrecords, FieldMetadata(alias="totalRecords")]
    total_amount: typing_extensions.Annotated[float, FieldMetadata(alias="totalAmount")] = pydantic.Field()
    """
    Total amount for the records.
    """

    totalactive: int = pydantic.Field()
    """
    Total number of active vCards.
    """

    totalamounteactive: float = pydantic.Field()
    """
    Total amount of active vCards.
    """

    totalbalanceactive: float = pydantic.Field()
    """
    Total balance of active vCards.
    """

    page_identifier: typing_extensions.Annotated[
        typing.Optional[PageIdentifier], FieldMetadata(alias="pageIdentifier")
    ] = None
    page_size: typing_extensions.Annotated[typing.Optional[Pagesize], FieldMetadata(alias="pageSize")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.accounttype import Accounttype
from ...types.created_at import CreatedAt
from ...types.dbaname import Dbaname
from ...types.entrypointfield import Entrypointfield
from ...types.external_paypoint_id import ExternalPaypointId
from ...types.legalname import Legalname
from ...types.netamountnullable import Netamountnullable
from ...types.order_id import OrderId
from ...types.org_parent_name import OrgParentName
from ...types.page_identifier import PageIdentifier
from ...types.query_payment_data import QueryPaymentData
from ...types.query_transaction_payor_data import QueryTransactionPayorData
from ...types.replyby import Replyby
from ...types.transaction_query_records import TransactionQueryRecords
from ...types.transaction_time import TransactionTime
from .charge_back_response import ChargeBackResponse
from .chargeback_message import ChargebackMessage


class ChargebackQueryRecords(UniversalBaseModel):
    id: typing_extensions.Annotated[int, FieldMetadata(alias="Id")] = pydantic.Field()
    """
    Identifier of chargeback or return.
    """

    chargeback_date: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="ChargebackDate")] = pydantic.Field()
    """
    Date of chargeback in format YYYY-MM-DD or MM/DD/YYYY.
    """

    case_number: typing_extensions.Annotated[str, FieldMetadata(alias="CaseNumber")] = pydantic.Field()
    """
    Number of case assigned to the chargeback.
    """

    reason_code: typing_extensions.Annotated[str, FieldMetadata(alias="ReasonCode")] = pydantic.Field()
    """
    R code for returned ACH or custom code identifying the reason.
    """

    reason: typing_extensions.Annotated[str, FieldMetadata(alias="Reason")] = pydantic.Field()
    """
    Text describing the chargeback or ACH return reason.
    """

    reference_number: typing_extensions.Annotated[str, FieldMetadata(alias="ReferenceNumber")] = pydantic.Field()
    """
    Processor reference number to the chargeback.
    """

    last_four: typing_extensions.Annotated[str, FieldMetadata(alias="LastFour")] = pydantic.Field()
    """
    Last 4 digits of card or bank account involved in chargeback or return.
    """

    account_type: typing_extensions.Annotated[Accounttype, FieldMetadata(alias="AccountType")]
    status: typing_extensions.Annotated[int, FieldMetadata(alias="Status")] = pydantic.Field()
    """
    Status for chargeback or ACH return
    
    - 0: Open (chargebacks only)
    - 1: Pending (chargebacks only)
    - 2: Closed-Won (chargebacks only)
    - 3: Closed-Lost (chargebacks only)
    - 4: ACH Return (ACH only)
    - 5: ACH Dispute, Not Authorized (ACH only)
    """

    method: typing_extensions.Annotated[str, FieldMetadata(alias="Method")] = pydantic.Field()
    """
    Type of payment vehicle: **ach** or **card**.
    """

    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="CreatedAt")] = pydantic.Field()
    """
    Timestamp when the register was created, in UTC.
    """

    reply_by: typing_extensions.Annotated[Replyby, FieldMetadata(alias="ReplyBy")]
    payment_trans_id: typing_extensions.Annotated[str, FieldMetadata(alias="PaymentTransId")] = pydantic.Field()
    """
    ReferenceId of the transaction in Payabli.
    """

    schedule_reference: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="ScheduleReference")] = (
        pydantic.Field(default=None)
    )
    """
    Reference to the subscription originating the transaction.
    """

    order_id: typing_extensions.Annotated[OrderId, FieldMetadata(alias="OrderId")]
    net_amount: typing_extensions.Annotated[Netamountnullable, FieldMetadata(alias="NetAmount")] = pydantic.Field()
    """
    Net amount in chargeback or ACH return.
    """

    transaction_time: typing_extensions.Annotated[TransactionTime, FieldMetadata(alias="TransactionTime")]
    customer: typing_extensions.Annotated[QueryTransactionPayorData, FieldMetadata(alias="Customer")]
    payment_data: typing_extensions.Annotated[QueryPaymentData, FieldMetadata(alias="PaymentData")]
    paypoint_legalname: typing_extensions.Annotated[Legalname, FieldMetadata(alias="PaypointLegalname")] = (
        pydantic.Field()
    )
    """
    The paypoint's legal name.
    """

    paypoint_dbaname: typing_extensions.Annotated[Dbaname, FieldMetadata(alias="PaypointDbaname")] = pydantic.Field()
    """
    The paypoint's DBA name.
    """

    parent_org_name: typing_extensions.Annotated[OrgParentName, FieldMetadata(alias="ParentOrgName")]
    parent_org_id: typing_extensions.Annotated[int, FieldMetadata(alias="ParentOrgId")] = pydantic.Field()
    """
    The ID of the parent organization.
    """

    paypoint_entryname: typing_extensions.Annotated[Entrypointfield, FieldMetadata(alias="PaypointEntryname")] = (
        pydantic.Field()
    )
    """
    The paypoint's entryname.
    """

    responses: typing_extensions.Annotated[typing.List[ChargeBackResponse], FieldMetadata(alias="Responses")] = (
        pydantic.Field()
    )
    """
    Chargeback response records.
    """

    transaction: typing_extensions.Annotated[TransactionQueryRecords, FieldMetadata(alias="Transaction")]
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None
    pageidentifier: typing.Optional[PageIdentifier] = None
    messages: typing.List[ChargebackMessage] = pydantic.Field()
    """
    Messages related to the chargeback.
    """

    service_group: typing_extensions.Annotated[str, FieldMetadata(alias="ServiceGroup")] = pydantic.Field()
    """
    Service group classification.
    """

    dispute_type: typing_extensions.Annotated[str, FieldMetadata(alias="DisputeType")] = pydantic.Field()
    """
    Type of dispute classification.
    """

    processor_name: typing_extensions.Annotated[str, FieldMetadata(alias="ProcessorName")] = pydantic.Field()
    """
    Name of the payment processor.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..money_out_types.types.auth_capture_payout_response import AuthCapturePayoutResponse
from ..money_out_types.types.authorize_payment_method import AuthorizePaymentMethod
from ..money_out_types.types.capture_all_out_response import CaptureAllOutResponse
from ..money_out_types.types.operation_result import OperationResult
from ..money_out_types.types.request_out_authorize_invoice_data import RequestOutAuthorizeInvoiceData
from ..money_out_types.types.request_out_authorize_payment_details import RequestOutAuthorizePaymentDetails
from ..money_out_types.types.request_out_authorize_vendor_data import RequestOutAuthorizeVendorData
from ..money_out_types.types.v_card_get_response import VCardGetResponse
from ..types.accountid import Accountid
from ..types.bill_detail_response import BillDetailResponse
from ..types.entrypointfield import Entrypointfield
from ..types.idempotency_key import IdempotencyKey
from ..types.order_id import OrderId
from ..types.orderdescription import Orderdescription
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payabli_api_response_0000 import PayabliApiResponse0000
from ..types.source import Source
from ..types.subdomain import Subdomain
from ..types.subscriptionid import Subscriptionid

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawMoneyOutClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def authorize_out(
        self,
        *,
        entry_point: Entrypointfield,
        payment_method: AuthorizePaymentMethod,
        payment_details: RequestOutAuthorizePaymentDetails,
        vendor_data: RequestOutAuthorizeVendorData,
        invoice_data: typing.Sequence[RequestOutAuthorizeInvoiceData],
        allow_duplicated_bills: typing.Optional[bool] = None,
        do_not_create_bills: typing.Optional[bool] = None,
        force_vendor_creation: typing.Optional[bool] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        source: typing.Optional[Source] = OMIT,
        order_id: typing.Optional[OrderId] = OMIT,
        order_description: typing.Optional[Orderdescription] = OMIT,
        account_id: typing.Optional[Accountid] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        subscription_id: typing.Optional[Subscriptionid] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AuthCapturePayoutResponse]:
        """
        Authorizes transaction for payout. Authorized transactions aren't flagged for settlement until captured. Use `referenceId` returned in the response to capture the transaction.

        Parameters
        ----------
        entry_point : Entrypointfield

        payment_method : AuthorizePaymentMethod

        payment_details : RequestOutAuthorizePaymentDetails
            Object containing payment details.

        vendor_data : RequestOutAuthorizeVendorData
            Object containing vendor data.

        invoice_data : typing.Sequence[RequestOutAuthorizeInvoiceData]
            Array of bills associated to the transaction

        allow_duplicated_bills : typing.Optional[bool]
            When `true`, the authorization bypasses the requirement for unique bills, identified by vendor invoice number. This allows you to make more than one payout authorization for a bill, like a split payment.

        do_not_create_bills : typing.Optional[bool]
            When `true`, Payabli won't automatically create a bill for this payout transaction.

        force_vendor_creation : typing.Optional[bool]
            When `true`, the request creates a new vendor record, regardless of whether the vendor already exists.

        idempotency_key : typing.Optional[IdempotencyKey]

        source : typing.Optional[Source]

        order_id : typing.Optional[OrderId]

        order_description : typing.Optional[Orderdescription]

        account_id : typing.Optional[Accountid]

        subdomain : typing.Optional[Subdomain]

        subscription_id : typing.Optional[Subscriptionid]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AuthCapturePayoutResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "MoneyOut/authorize",
            method="POST",
            params={
                "allowDuplicatedBills": allow_duplicated_bills,
                "doNotCreateBills": do_not_create_bills,
                "forceVendorCreation": force_vendor_creation,
            },
            json={
                "entryPoint": entry_point,
                "source": source,
                "orderId": order_id,
                "orderDescription": order_description,
                "paymentMethod": convert_and_respect_annotation_metadata(
                    object_=payment_method, annotation=AuthorizePaymentMethod, direction="write"
                ),
                "paymentDetails": convert_and_respect_annotation_metadata(
                    object_=payment_details, annotation=RequestOutAuthorizePaymentDetails, direction="write"
                ),
                "vendorData": convert_and_respect_annotation_metadata(
                    object_=vendor_data, annotation=RequestOutAuthorizeVendorData, direction="write"
                ),
                "invoiceData": convert_and_respect_annotation_metadata(
                    object_=invoice_data, annotation=typing.Sequence[RequestOutAuthorizeInvoiceData], direction="write"
                ),
                "accountId": account_id,
                "subdomain": subdomain,
                "subscriptionId": subscription_id,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AuthCapturePayoutResponse,
                    parse_obj_as(
                        type_=AuthCapturePayoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def cancel_all_out(
        self, *, request: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CaptureAllOutResponse]:
        """
        Cancels an array of payout transactions.

        Parameters
        ----------
        request : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CaptureAllOutResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "MoneyOut/cancelAll",
            method="POST",
            json=request,
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CaptureAllOutResponse,
                    parse_obj_as(
                        type_=CaptureAllOutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def cancel_out(
        self, reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponse0000]:
        """
        Cancel a payout transaction by ID.

        Parameters
        ----------
        reference_id : str
            The ID for the payout transaction.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponse0000]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"MoneyOut/cancel/{jsonable_encoder(reference_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse0000,
                    parse_obj_as(
                        type_=PayabliApiResponse0000,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def capture_all_out(
        self,
        *,
        request: typing.Sequence[str],
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CaptureAllOutResponse]:
        """
        Captures an array of authorized payout transactions for settlement.

        Parameters
        ----------
        request : typing.Sequence[str]

        idempotency_key : typing.Optional[IdempotencyKey]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CaptureAllOutResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "MoneyOut/captureAll",
            method="POST",
            json=request,
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CaptureAllOutResponse,
                    parse_obj_as(
                        type_=CaptureAllOutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def capture_out(
        self,
        reference_id: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AuthCapturePayoutResponse]:
        """
        Captures a single authorized payout transaction by ID.

        Parameters
        ----------
        reference_id : str
            The ID for the payout transaction.

        idempotency_key : typing.Optional[IdempotencyKey]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AuthCapturePayoutResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"MoneyOut/capture/{jsonable_encoder(reference_id)}",
            method="GET",
            headers={
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AuthCapturePayoutResponse,
                    parse_obj_as(
                        type_=AuthCapturePayoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def payout_details(
        self, trans_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[BillDetailResponse]:
        """
        Returns details for a processed money out transaction.

        Parameters
        ----------
        trans_id : str
            ReferenceId for the transaction (PaymentId).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BillDetailResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"MoneyOut/details/{jsonable_encoder(trans_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillDetailResponse,
                    parse_obj_as(
                        type_=BillDetailResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def v_card_get(
        self, card_token: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[VCardGetResponse]:
        """
        Retrieves vCard details for a single card in an entrypoint.

        Parameters
        ----------
        card_token : str
            ID for a virtual card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VCardGetResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"MoneyOut/vcard/{jsonable_encoder(card_token)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VCardGetResponse,
                    parse_obj_as(
                        type_=VCardGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def send_v_card_link(
        self, *, trans_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[OperationResult]:
        """
        Sends a virtual card link via email to the vendor associated with the `transId`.

        Parameters
        ----------
        trans_id : str
            The transaction ID of the virtual card payout. The ID is returned as `ReferenceId` in the response when you authorize a payout with POST /MoneyOut/authorize.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[OperationResult]
        """
        _response = self._client_wrapper.httpx_client.request(
            "vcard/send-card-link",
            method="POST",
            json={
                "transId": trans_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OperationResult,
                    parse_obj_as(
                        type_=OperationResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_check_image(
        self, asset_name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[str]:
        """
        Retrieve the image of a check associated with a processed transaction.
        The check image is returned in the response body as a base64-encoded string.
        The check image is only available for payouts that have been processed.

        Parameters
        ----------
        asset_name : str
            Name of the check asset to retrieve. This is returned as `filename` in the `CheckData` object
            in the response when you make a GET request to `/MoneyOut/details/{transId}`.
            ```
                "CheckData": {
                  "ftype": "PDF",
                  "filename": "check133832686289732320_01JKBNZ5P32JPTZY8XXXX000000.pdf",
                  "furl": "",
                  "fContent": ""
              }
            ```

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[str]
            A base64-encoded string of the check image.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"MoneyOut/checkimage/{jsonable_encoder(asset_name)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    str,
                    parse_obj_as(
                        type_=str,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawMoneyOutClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def authorize_out(
        self,
        *,
        entry_point: Entrypointfield,
        payment_method: AuthorizePaymentMethod,
        payment_details: RequestOutAuthorizePaymentDetails,
        vendor_data: RequestOutAuthorizeVendorData,
        invoice_data: typing.Sequence[RequestOutAuthorizeInvoiceData],
        allow_duplicated_bills: typing.Optional[bool] = None,
        do_not_create_bills: typing.Optional[bool] = None,
        force_vendor_creation: typing.Optional[bool] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        source: typing.Optional[Source] = OMIT,
        order_id: typing.Optional[OrderId] = OMIT,
        order_description: typing.Optional[Orderdescription] = OMIT,
        account_id: typing.Optional[Accountid] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        subscription_id: typing.Optional[Subscriptionid] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AuthCapturePayoutResponse]:
        """
        Authorizes transaction for payout. Authorized transactions aren't flagged for settlement until captured. Use `referenceId` returned in the response to capture the transaction.

        Parameters
        ----------
        entry_point : Entrypointfield

        payment_method : AuthorizePaymentMethod

        payment_details : RequestOutAuthorizePaymentDetails
            Object containing payment details.

        vendor_data : RequestOutAuthorizeVendorData
            Object containing vendor data.

        invoice_data : typing.Sequence[RequestOutAuthorizeInvoiceData]
            Array of bills associated to the transaction

        allow_duplicated_bills : typing.Optional[bool]
            When `true`, the authorization bypasses the requirement for unique bills, identified by vendor invoice number. This allows you to make more than one payout authorization for a bill, like a split payment.

        do_not_create_bills : typing.Optional[bool]
            When `true`, Payabli won't automatically create a bill for this payout transaction.

        force_vendor_creation : typing.Optional[bool]
            When `true`, the request creates a new vendor record, regardless of whether the vendor already exists.

        idempotency_key : typing.Optional[IdempotencyKey]

        source : typing.Optional[Source]

        order_id : typing.Optional[OrderId]

        order_description : typing.Optional[Orderdescription]

        account_id : typing.Optional[Accountid]

        subdomain : typing.Optional[Subdomain]

        subscription_id : typing.Optional[Subscriptionid]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AuthCapturePayoutResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "MoneyOut/authorize",
            method="POST",
            params={
                "allowDuplicatedBills": allow_duplicated_bills,
                "doNotCreateBills": do_not_create_bills,
                "forceVendorCreation": force_vendor_creation,
            },
            json={
                "entryPoint": entry_point,
                "source": source,
                "orderId": order_id,
                "orderDescription": order_description,
                "paymentMethod": convert_and_respect_annotation_metadata(
                    object_=payment_method, annotation=AuthorizePaymentMethod, direction="write"
                ),
                "paymentDetails": convert_and_respect_annotation_metadata(
                    object_=payment_details, annotation=RequestOutAuthorizePaymentDetails, direction="write"
                ),
                "vendorData": convert_and_respect_annotation_metadata(
                    object_=vendor_data, annotation=RequestOutAuthorizeVendorData, direction="write"
                ),
                "invoiceData": convert_and_respect_annotation_metadata(
                    object_=invoice_data, annotation=typing.Sequence[RequestOutAuthorizeInvoiceData], direction="write"
                ),
                "accountId": account_id,
                "subdomain": subdomain,
                "subscriptionId": subscription_id,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AuthCapturePayoutResponse,
                    parse_obj_as(
                        type_=AuthCapturePayoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def cancel_all_out(
        self, *, request: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CaptureAllOutResponse]:
        """
        Cancels an array of payout transactions.

        Parameters
        ----------
        request : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CaptureAllOutResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "MoneyOut/cancelAll",
            method="POST",
            json=request,
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CaptureAllOutResponse,
                    parse_obj_as(
                        type_=CaptureAllOutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def cancel_out(
        self, reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponse0000]:
        """
        Cancel a payout transaction by ID.

        Parameters
        ----------
        reference_id : str
            The ID for the payout transaction.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponse0000]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"MoneyOut/cancel/{jsonable_encoder(reference_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse0000,
                    parse_obj_as(
                        type_=PayabliApiResponse0000,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def capture_all_out(
        self,
        *,
        request: typing.Sequence[str],
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CaptureAllOutResponse]:
        """
        Captures an array of authorized payout transactions for settlement.

        Parameters
        ----------
        request : typing.Sequence[str]

        idempotency_key : typing.Optional[IdempotencyKey]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CaptureAllOutResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "MoneyOut/captureAll",
            method="POST",
            json=request,
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CaptureAllOutResponse,
                    parse_obj_as(
                        type_=CaptureAllOutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def capture_out(
        self,
        reference_id: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AuthCapturePayoutResponse]:
        """
        Captures a single authorized payout transaction by ID.

        Parameters
        ----------
        reference_id : str
            The ID for the payout transaction.

        idempotency_key : typing.Optional[IdempotencyKey]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AuthCapturePayoutResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"MoneyOut/capture/{jsonable_encoder(reference_id)}",
            method="GET",
            headers={
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AuthCapturePayoutResponse,
                    parse_obj_as(
                        type_=AuthCapturePayoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def payout_details(
        self, trans_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[BillDetailResponse]:
        """
        Returns details for a processed money out transaction.

        Parameters
        ----------
        trans_id : str
            ReferenceId for the transaction (PaymentId).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BillDetailResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"MoneyOut/details/{jsonable_encoder(trans_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillDetailResponse,
                    parse_obj_as(
                        type_=BillDetailResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def v_card_get(
        self, card_token: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[VCardGetResponse]:
        """
        Retrieves vCard details for a single card in an entrypoint.

        Parameters
        ----------
        card_token : str
            ID for a virtual card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VCardGetResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"MoneyOut/vcard/{jsonable_encoder(card_token)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VCardGetResponse,
                    parse_obj_as(
                        type_=VCardGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def send_v_card_link(
        self, *, trans_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[OperationResult]:
        """
        Sends a virtual card link via email to the vendor associated with the `transId`.

        Parameters
        ----------
        trans_id : str
            The transaction ID of the virtual card payout. The ID is returned as `ReferenceId` in the response when you authorize a payout with POST /MoneyOut/authorize.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[OperationResult]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "vcard/send-card-link",
            method="POST",
            json={
                "transId": trans_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OperationResult,
                    parse_obj_as(
                        type_=OperationResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_check_image(
        self, asset_name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[str]:
        """
        Retrieve the image of a check associated with a processed transaction.
        The check image is returned in the response body as a base64-encoded string.
        The check image is only available for payouts that have been processed.

        Parameters
        ----------
        asset_name : str
            Name of the check asset to retrieve. This is returned as `filename` in the `CheckData` object
            in the response when you make a GET request to `/MoneyOut/details/{transId}`.
            ```
                "CheckData": {
                  "ftype": "PDF",
                  "filename": "check133832686289732320_01JKBNZ5P32JPTZY8XXXX000000.pdf",
                  "furl": "",
                  "fContent": ""
              }
            ```

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[str]
            A base64-encoded string of the check image.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"MoneyOut/checkimage/{jsonable_encoder(asset_name)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    str,
                    parse_obj_as(
                        type_=str,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .transaction_time import TransactionTime


class QueryCFeeTransaction(UniversalBaseModel):
    c_fee_transid: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="cFeeTransid")] = None
    fee_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="feeAmount")] = None
    operation: typing.Optional[str] = None
    refund_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="refundId")] = None
    response_data: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="responseData")
    ] = None
    settlement_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="settlementStatus")] = None
    transaction_time: typing_extensions.Annotated[
        typing.Optional[TransactionTime], FieldMetadata(alias="transactionTime")
    ] = None
    trans_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="transStatus")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data_string import AdditionalDataString
from .email import Email


class Owners(UniversalBaseModel):
    ownername: typing.Optional[str] = pydantic.Field(default=None)
    """
    Person who is registered as the beneficial owner of the business. This is a combination of first and last name.
    """

    ownertitle: typing.Optional[str] = pydantic.Field(default=None)
    """
    The job title of the person such as CEO or director.
    """

    ownerpercent: typing.Optional[int] = pydantic.Field(default=None)
    """
    Percentage of ownership the person holds, in integer format.
    """

    ownerssn: typing.Optional[str] = pydantic.Field(default=None)
    """
    The relevant identifier for the person such as a Social Security Number.
    """

    ownerdob: typing.Optional[str] = pydantic.Field(default=None)
    """
    Owner's date of birth.
    """

    ownerphone_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ownerphone1")] = (
        pydantic.Field(default=None)
    )
    """
    Owner phone 1.
    """

    ownerphone_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ownerphone2")] = (
        pydantic.Field(default=None)
    )
    """
    Owner phone 2.
    """

    owneremail: typing.Optional[Email] = pydantic.Field(default=None)
    """
    Owner email.
    """

    ownerdriver: typing.Optional[str] = pydantic.Field(default=None)
    """
    Owner driver's license ID number. Payabli strongly recommends including this.
    """

    oaddress: typing.Optional[str] = pydantic.Field(default=None)
    """
    Owner street address. This must be the physical address of the owner, not a P.O. box.
    """

    ocity: typing.Optional[str] = pydantic.Field(default=None)
    """
    Owner address city.
    """

    ocountry: typing.Optional[str] = pydantic.Field(default=None)
    """
    Owner address country in ISO-3166-1 alpha 2 format. Check out https://en.wikipedia.org/wiki/ISO_3166-1 for reference.
    """

    odriverstate: typing.Optional[str] = pydantic.Field(default=None)
    """
    Owner driver's license State. Payabli strongly recommends including this.
    """

    ostate: typing.Optional[str] = pydantic.Field(default=None)
    """
    Owner address state.
    """

    ozip: typing.Optional[str] = pydantic.Field(default=None)
    """
    Owner address ZIP.
    """

    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString], FieldMetadata(alias="additionalData")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ach_holder import AchHolder


class Check(UniversalBaseModel):
    ach_holder: typing_extensions.Annotated[AchHolder, FieldMetadata(alias="achHolder")] = pydantic.Field()
    """
    The checking accountholder's name.
    """

    method: typing.Literal["check"] = pydantic.Field(default="check")
    """
    Method to use for the transaction. Use `check` for a paper check transaction. When the method is `check`, then `paymentDetails.checkNumber` is required.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .key_value_array import KeyValueArray
from .notification_log import NotificationLog
from .string_string_key_value_pair import StringStringKeyValuePair


class NotificationLogDetail(NotificationLog):
    web_headers: typing_extensions.Annotated[
        typing.Optional[typing.List[StringStringKeyValuePair]], FieldMetadata(alias="webHeaders")
    ] = None
    response_headers: typing_extensions.Annotated[
        typing.Optional[typing.List[KeyValueArray]], FieldMetadata(alias="responseHeaders")
    ] = None
    response_content: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="responseContent")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .page_identifier import PageIdentifier
from .pagesize import Pagesize
from .totalrecords import Totalrecords


class QuerySummary(UniversalBaseModel):
    page_identifier: typing_extensions.Annotated[
        typing.Optional[PageIdentifier], FieldMetadata(alias="pageIdentifier")
    ] = None
    page_size: typing_extensions.Annotated[typing.Optional[Pagesize], FieldMetadata(alias="pageSize")] = None
    total_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="totalAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Total amount for the records.
    """

    total_net_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="totalNetAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Total net amount for the records.
    """

    total_pages: typing_extensions.Annotated[typing.Optional[Totalrecords], FieldMetadata(alias="totalPages")] = None
    total_records: typing_extensions.Annotated[typing.Optional[Totalrecords], FieldMetadata(alias="totalRecords")] = (
        None
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .template_element import TemplateElement
from .visible import Visible


class BankSection(UniversalBaseModel):
    """
    Details about a bank account.
    """

    visible: typing.Optional[Visible] = None
    account_number: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="accountNumber")
    ] = None
    account_type: typing_extensions.Annotated[typing.Optional[TemplateElement], FieldMetadata(alias="accountType")] = (
        None
    )
    bank_name: typing_extensions.Annotated[typing.Optional[TemplateElement], FieldMetadata(alias="bankName")] = None
    routing_number: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="routingNumber")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.additional_data import AdditionalData
from ..types.idempotency_key import IdempotencyKey
from ..types.page_content import PageContent
from ..types.page_identifier import PageIdentifier
from ..types.page_setting import PageSetting
from ..types.payabli_api_response_00_responsedatanonobject import PayabliApiResponse00Responsedatanonobject
from ..types.payabli_credentials import PayabliCredentials
from ..types.payabli_pages import PayabliPages
from ..types.receipt_content import ReceiptContent
from ..types.subdomain import Subdomain
from .raw_client import AsyncRawHostedPaymentPagesClient, RawHostedPaymentPagesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class HostedPaymentPagesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawHostedPaymentPagesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawHostedPaymentPagesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawHostedPaymentPagesClient
        """
        return self._raw_client

    def load_page(
        self, entry: str, subdomain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliPages:
        """
        Loads all of a payment page's details including `pageIdentifier` and `validationCode`. This endpoint requires an `application` API token.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain : str
            Payment page identifier. The subdomain value is the last part of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliPages
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.hosted_payment_pages.load_page(
            entry="8cfec329267",
            subdomain="pay-your-fees-1",
        )
        """
        _response = self._raw_client.load_page(entry, subdomain, request_options=request_options)
        return _response.data

    def new_page(
        self,
        entry: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        credentials: typing.Optional[typing.Sequence[PayabliCredentials]] = OMIT,
        last_access: typing.Optional[dt.datetime] = OMIT,
        page_content: typing.Optional[PageContent] = OMIT,
        page_identifier: typing.Optional[PageIdentifier] = OMIT,
        page_settings: typing.Optional[PageSetting] = OMIT,
        published: typing.Optional[int] = OMIT,
        receipt_content: typing.Optional[ReceiptContent] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        validation_code: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse00Responsedatanonobject:
        """

        Creates a new payment page for a paypoint.
        Note: this operation doesn't create a new paypoint, just a payment page for an existing paypoint. Paypoints are created by the Payabli team when a boarding application is approved.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        idempotency_key : typing.Optional[IdempotencyKey]

        additional_data : typing.Optional[AdditionalData]

        credentials : typing.Optional[typing.Sequence[PayabliCredentials]]
            Array of credential objects with active services for the page

        last_access : typing.Optional[dt.datetime]
            Timestamp of last access to page structure

        page_content : typing.Optional[PageContent]
            Sections of page

        page_identifier : typing.Optional[PageIdentifier]

        page_settings : typing.Optional[PageSetting]
            Settings of page

        published : typing.Optional[int]
            Flag indicating if page is active to accept payments. `0` for false, `1` for true.

        receipt_content : typing.Optional[ReceiptContent]
            Sections of payment receipt

        subdomain : typing.Optional[Subdomain]
            Page identifier. Must be unique in platform.

        total_amount : typing.Optional[float]
            Total amount to pay in this page

        validation_code : typing.Optional[str]
            Base64 encoded image of CAPTCHA associated to this page load

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.hosted_payment_pages.new_page(
            entry="8cfec329267",
            idempotency_key="6B29FC40-CA47-1067-B31D-00DD010662DA",
        )
        """
        _response = self._raw_client.new_page(
            entry,
            idempotency_key=idempotency_key,
            additional_data=additional_data,
            credentials=credentials,
            last_access=last_access,
            page_content=page_content,
            page_identifier=page_identifier,
            page_settings=page_settings,
            published=published,
            receipt_content=receipt_content,
            subdomain=subdomain,
            total_amount=total_amount,
            validation_code=validation_code,
            request_options=request_options,
        )
        return _response.data

    def save_page(
        self,
        entry: str,
        subdomain_: str,
        *,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        credentials: typing.Optional[typing.Sequence[PayabliCredentials]] = OMIT,
        last_access: typing.Optional[dt.datetime] = OMIT,
        page_content: typing.Optional[PageContent] = OMIT,
        page_identifier: typing.Optional[PageIdentifier] = OMIT,
        page_settings: typing.Optional[PageSetting] = OMIT,
        published: typing.Optional[int] = OMIT,
        receipt_content: typing.Optional[ReceiptContent] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        validation_code: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Updates a payment page in a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain_ : str
            Payment page identifier. The subdomain value is the last part of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        additional_data : typing.Optional[AdditionalData]

        credentials : typing.Optional[typing.Sequence[PayabliCredentials]]
            Array of credential objects with active services for the page

        last_access : typing.Optional[dt.datetime]
            Timestamp of last access to page structure

        page_content : typing.Optional[PageContent]
            Sections of page

        page_identifier : typing.Optional[PageIdentifier]

        page_settings : typing.Optional[PageSetting]
            Settings of page

        published : typing.Optional[int]
            Flag indicating if page is active to accept payments. `0` for false, `1` for true.

        receipt_content : typing.Optional[ReceiptContent]
            Sections of payment receipt

        subdomain : typing.Optional[Subdomain]
            Page identifier. Must be unique in platform.

        total_amount : typing.Optional[float]
            Total amount to pay in this page

        validation_code : typing.Optional[str]
            Base64 encoded image of CAPTCHA associated to this page load

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.hosted_payment_pages.save_page(
            entry="8cfec329267",
            subdomain_="pay-your-fees-1",
        )
        """
        _response = self._raw_client.save_page(
            entry,
            subdomain_,
            additional_data=additional_data,
            credentials=credentials,
            last_access=last_access,
            page_content=page_content,
            page_identifier=page_identifier,
            page_settings=page_settings,
            published=published,
            receipt_content=receipt_content,
            subdomain=subdomain,
            total_amount=total_amount,
            validation_code=validation_code,
            request_options=request_options,
        )
        return _response.data


class AsyncHostedPaymentPagesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawHostedPaymentPagesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawHostedPaymentPagesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawHostedPaymentPagesClient
        """
        return self._raw_client

    async def load_page(
        self, entry: str, subdomain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliPages:
        """
        Loads all of a payment page's details including `pageIdentifier` and `validationCode`. This endpoint requires an `application` API token.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain : str
            Payment page identifier. The subdomain value is the last part of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliPages
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.hosted_payment_pages.load_page(
                entry="8cfec329267",
                subdomain="pay-your-fees-1",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.load_page(entry, subdomain, request_options=request_options)
        return _response.data

    async def new_page(
        self,
        entry: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        credentials: typing.Optional[typing.Sequence[PayabliCredentials]] = OMIT,
        last_access: typing.Optional[dt.datetime] = OMIT,
        page_content: typing.Optional[PageContent] = OMIT,
        page_identifier: typing.Optional[PageIdentifier] = OMIT,
        page_settings: typing.Optional[PageSetting] = OMIT,
        published: typing.Optional[int] = OMIT,
        receipt_content: typing.Optional[ReceiptContent] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        validation_code: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse00Responsedatanonobject:
        """

        Creates a new payment page for a paypoint.
        Note: this operation doesn't create a new paypoint, just a payment page for an existing paypoint. Paypoints are created by the Payabli team when a boarding application is approved.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        idempotency_key : typing.Optional[IdempotencyKey]

        additional_data : typing.Optional[AdditionalData]

        credentials : typing.Optional[typing.Sequence[PayabliCredentials]]
            Array of credential objects with active services for the page

        last_access : typing.Optional[dt.datetime]
            Timestamp of last access to page structure

        page_content : typing.Optional[PageContent]
            Sections of page

        page_identifier : typing.Optional[PageIdentifier]

        page_settings : typing.Optional[PageSetting]
            Settings of page

        published : typing.Optional[int]
            Flag indicating if page is active to accept payments. `0` for false, `1` for true.

        receipt_content : typing.Optional[ReceiptContent]
            Sections of payment receipt

        subdomain : typing.Optional[Subdomain]
            Page identifier. Must be unique in platform.

        total_amount : typing.Optional[float]
            Total amount to pay in this page

        validation_code : typing.Optional[str]
            Base64 encoded image of CAPTCHA associated to this page load

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.hosted_payment_pages.new_page(
                entry="8cfec329267",
                idempotency_key="6B29FC40-CA47-1067-B31D-00DD010662DA",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.new_page(
            entry,
            idempotency_key=idempotency_key,
            additional_data=additional_data,
            credentials=credentials,
            last_access=last_access,
            page_content=page_content,
            page_identifier=page_identifier,
            page_settings=page_settings,
            published=published,
            receipt_content=receipt_content,
            subdomain=subdomain,
            total_amount=total_amount,
            validation_code=validation_code,
            request_options=request_options,
        )
        return _response.data

    async def save_page(
        self,
        entry: str,
        subdomain_: str,
        *,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        credentials: typing.Optional[typing.Sequence[PayabliCredentials]] = OMIT,
        last_access: typing.Optional[dt.datetime] = OMIT,
        page_content: typing.Optional[PageContent] = OMIT,
        page_identifier: typing.Optional[PageIdentifier] = OMIT,
        page_settings: typing.Optional[PageSetting] = OMIT,
        published: typing.Optional[int] = OMIT,
        receipt_content: typing.Optional[ReceiptContent] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        validation_code: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Updates a payment page in a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain_ : str
            Payment page identifier. The subdomain value is the last part of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        additional_data : typing.Optional[AdditionalData]

        credentials : typing.Optional[typing.Sequence[PayabliCredentials]]
            Array of credential objects with active services for the page

        last_access : typing.Optional[dt.datetime]
            Timestamp of last access to page structure

        page_content : typing.Optional[PageContent]
            Sections of page

        page_identifier : typing.Optional[PageIdentifier]

        page_settings : typing.Optional[PageSetting]
            Settings of page

        published : typing.Optional[int]
            Flag indicating if page is active to accept payments. `0` for false, `1` for true.

        receipt_content : typing.Optional[ReceiptContent]
            Sections of payment receipt

        subdomain : typing.Optional[Subdomain]
            Page identifier. Must be unique in platform.

        total_amount : typing.Optional[float]
            Total amount to pay in this page

        validation_code : typing.Optional[str]
            Base64 encoded image of CAPTCHA associated to this page load

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.hosted_payment_pages.save_page(
                entry="8cfec329267",
                subdomain_="pay-your-fees-1",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.save_page(
            entry,
            subdomain_,
            additional_data=additional_data,
            credentials=credentials,
            last_access=last_access,
            page_content=page_content,
            page_identifier=page_identifier,
            page_settings=page_settings,
            published=published,
            receipt_content=receipt_content,
            subdomain=subdomain,
            total_amount=total_amount,
            validation_code=validation_code,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .transaction_query_records import TransactionQueryRecords


class CustomerSummaryRecord(UniversalBaseModel):
    numberof_transactions: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="numberofTransactions")
    ] = pydantic.Field(default=None)
    """
    Number total of transactions or payments
    """

    recent_transactions: typing_extensions.Annotated[
        typing.Optional[typing.List[TransactionQueryRecords]], FieldMetadata(alias="recentTransactions")
    ] = pydantic.Field(default=None)
    """
    List of more recent 5 transactions belonging to the customer
    """

    total_amount_transactions: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="totalAmountTransactions")
    ] = pydantic.Field(default=None)
    """
    Total amount in transactions
    """

    total_net_amount_transactions: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="totalNetAmountTransactions")
    ] = pydantic.Field(default=None)
    """
    Total net amount in transactions
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .transfer import Transfer
from .transfer_summary import TransferSummary


class TransferQueryResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from payabli import (
        GeneralEvents,
        Transfer,
        TransferBankAccount,
        TransferQueryResponse,
        TransferSummary,
    )

    TransferQueryResponse(
        records=[
            Transfer(
                transfer_id=79851,
                paypoint_id=705,
                batch_number="split_705_gp_11-16-2024",
                batch_currency="USD",
                batch_records=1,
                transfer_identifier="bbcbfed7-e535-45fe-8d62-000000",
                batch_id=111430,
                paypoint_entry_name="47ae3de37",
                paypoint_legal_name="Gruzya Outdoor Outfitters LLC",
                paypoint_dba_name="Gruzya Outdoor Outfitters",
                paypoint_logo="https://example.com/logo.png",
                parent_org_name="Pilgrim Planner",
                parent_org_id=12345,
                parent_org_logo="https://example.com/parent-logo.png",
                external_paypoint_id="ext-12345",
                bank_account=TransferBankAccount(
                    account_number="****1234",
                    routing_number="123456789",
                ),
                transfer_date="2024-11-17T08:20:07.288+00:00",
                processor="gp",
                transfer_status=2,
                gross_amount=1029.0,
                charge_back_amount=25.0,
                returned_amount=0.0,
                hold_amount=0.0,
                released_amount=0.0,
                billing_fees_amount=0.0,
                third_party_paid_amount=0.0,
                adjustments_amount=0.0,
                net_transfer_amount=1004.0,
                events_data=[
                    GeneralEvents(
                        description="Transfer Created",
                        event_time=datetime.datetime.fromisoformat(
                            "2024-11-16 08:15:33.436000+00:00",
                        ),
                        source="worker",
                    )
                ],
                messages=[],
            )
        ],
        summary=TransferSummary(
            total_pages=1,
            total_records=2,
            page_size=20,
        ),
    )
    """

    records: typing_extensions.Annotated[typing.List[Transfer], FieldMetadata(alias="Records")]
    summary: typing_extensions.Annotated[TransferSummary, FieldMetadata(alias="Summary")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .link_data import LinkData


class CList(UniversalBaseModel):
    contact_email: typing_extensions.Annotated[typing.Optional[LinkData], FieldMetadata(alias="contactEmail")] = None
    contact_name: typing_extensions.Annotated[typing.Optional[LinkData], FieldMetadata(alias="contactName")] = None
    contact_phone: typing_extensions.Annotated[typing.Optional[LinkData], FieldMetadata(alias="contactPhone")] = None
    contact_title: typing_extensions.Annotated[typing.Optional[LinkData], FieldMetadata(alias="contactTitle")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.entrypointfield import Entrypointfield
from ..types.file_content_ftype import FileContentFtype
from ..types.payabli_api_response_00_responsedatanonobject import PayabliApiResponse00Responsedatanonobject
from ..types.payabli_api_response_generic_2_part import PayabliApiResponseGeneric2Part
from ..types.payabli_pages import PayabliPages
from ..types.settings_query_record import SettingsQueryRecord
from .raw_client import AsyncRawPaypointClient, RawPaypointClient
from .types.get_basic_entry_by_id_response import GetBasicEntryByIdResponse
from .types.get_basic_entry_response import GetBasicEntryResponse
from .types.get_entry_config_response import GetEntryConfigResponse
from .types.migrate_paypoint_response import MigratePaypointResponse
from .types.notification_request import NotificationRequest

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PaypointClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPaypointClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPaypointClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPaypointClient
        """
        return self._raw_client

    def get_basic_entry(
        self, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBasicEntryResponse:
        """
        Gets the basic details for a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBasicEntryResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.paypoint.get_basic_entry(
            entry="8cfec329267",
        )
        """
        _response = self._raw_client.get_basic_entry(entry, request_options=request_options)
        return _response.data

    def get_basic_entry_by_id(
        self, id_paypoint: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBasicEntryByIdResponse:
        """
        Retrieves the basic details for a paypoint by ID.

        Parameters
        ----------
        id_paypoint : str
            Paypoint ID. You can find this value by querying `/api/Query/paypoints/{orgId}`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBasicEntryByIdResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.paypoint.get_basic_entry_by_id(
            id_paypoint="198",
        )
        """
        _response = self._raw_client.get_basic_entry_by_id(id_paypoint, request_options=request_options)
        return _response.data

    def get_entry_config(
        self,
        entry: str,
        *,
        entrypages: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetEntryConfigResponse:
        """
        Gets the details for a single paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        entrypages : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEntryConfigResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.paypoint.get_entry_config(
            entry="8cfec329267",
        )
        """
        _response = self._raw_client.get_entry_config(entry, entrypages=entrypages, request_options=request_options)
        return _response.data

    def get_page(
        self, entry: str, subdomain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliPages:
        """
        Gets the details for single payment page for a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain : str
            Payment page identifier. The subdomain value is the last portion of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliPages
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.paypoint.get_page(
            entry="8cfec329267",
            subdomain="pay-your-fees-1",
        )
        """
        _response = self._raw_client.get_page(entry, subdomain, request_options=request_options)
        return _response.data

    def remove_page(
        self, entry: str, subdomain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseGeneric2Part:
        """
        Deletes a payment page in a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain : str
            Payment page identifier. The subdomain value is the last portion of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseGeneric2Part
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.paypoint.remove_page(
            entry="8cfec329267",
            subdomain="pay-your-fees-1",
        )
        """
        _response = self._raw_client.remove_page(entry, subdomain, request_options=request_options)
        return _response.data

    def save_logo(
        self,
        entry: str,
        *,
        f_content: typing.Optional[str] = OMIT,
        filename: typing.Optional[str] = OMIT,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        furl: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Updates a paypoint logo.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        f_content : typing.Optional[str]
            Content of file, Base64-encoded. Ignored if furl is specified. Max upload size is 30 MB.

        filename : typing.Optional[str]
            The name of the attached file.

        ftype : typing.Optional[FileContentFtype]
            The MIME type of the file (if content is provided)

        furl : typing.Optional[str]
            Optional URL provided to show or download the file remotely

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.paypoint.save_logo(
            entry="8cfec329267",
        )
        """
        _response = self._raw_client.save_logo(
            entry, f_content=f_content, filename=filename, ftype=ftype, furl=furl, request_options=request_options
        )
        return _response.data

    def settings_page(
        self, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SettingsQueryRecord:
        """
        Retrieves an paypoint's basic settings like custom fields, identifiers, and invoicing settings.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SettingsQueryRecord
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.paypoint.settings_page(
            entry="8cfec329267",
        )
        """
        _response = self._raw_client.settings_page(entry, request_options=request_options)
        return _response.data

    def migrate(
        self,
        *,
        entry_point: Entrypointfield,
        new_parent_organization_id: int,
        notification_request: typing.Optional[NotificationRequest] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MigratePaypointResponse:
        """
        Migrates a paypoint to a new parent organization.

        Parameters
        ----------
        entry_point : Entrypointfield

        new_parent_organization_id : int
            The ID for the paypoint's new parent organization.

        notification_request : typing.Optional[NotificationRequest]
            Optional notification request object for a webhook

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MigratePaypointResponse
            Success

        Examples
        --------
        from payabli import payabli
        from payabli.paypoint import NotificationRequest, WebHeaderParameter

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.paypoint.migrate(
            entry_point="473abc123def",
            new_parent_organization_id=123,
            notification_request=NotificationRequest(
                notification_url="https://webhook-test.yoursie.com",
                web_header_parameters=[
                    WebHeaderParameter(
                        key="testheader",
                        value="1234567890",
                    )
                ],
            ),
        )
        """
        _response = self._raw_client.migrate(
            entry_point=entry_point,
            new_parent_organization_id=new_parent_organization_id,
            notification_request=notification_request,
            request_options=request_options,
        )
        return _response.data


class AsyncPaypointClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPaypointClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPaypointClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPaypointClient
        """
        return self._raw_client

    async def get_basic_entry(
        self, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBasicEntryResponse:
        """
        Gets the basic details for a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBasicEntryResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.paypoint.get_basic_entry(
                entry="8cfec329267",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_basic_entry(entry, request_options=request_options)
        return _response.data

    async def get_basic_entry_by_id(
        self, id_paypoint: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBasicEntryByIdResponse:
        """
        Retrieves the basic details for a paypoint by ID.

        Parameters
        ----------
        id_paypoint : str
            Paypoint ID. You can find this value by querying `/api/Query/paypoints/{orgId}`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBasicEntryByIdResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.paypoint.get_basic_entry_by_id(
                id_paypoint="198",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_basic_entry_by_id(id_paypoint, request_options=request_options)
        return _response.data

    async def get_entry_config(
        self,
        entry: str,
        *,
        entrypages: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetEntryConfigResponse:
        """
        Gets the details for a single paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        entrypages : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEntryConfigResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.paypoint.get_entry_config(
                entry="8cfec329267",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_entry_config(
            entry, entrypages=entrypages, request_options=request_options
        )
        return _response.data

    async def get_page(
        self, entry: str, subdomain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliPages:
        """
        Gets the details for single payment page for a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain : str
            Payment page identifier. The subdomain value is the last portion of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliPages
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.paypoint.get_page(
                entry="8cfec329267",
                subdomain="pay-your-fees-1",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_page(entry, subdomain, request_options=request_options)
        return _response.data

    async def remove_page(
        self, entry: str, subdomain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseGeneric2Part:
        """
        Deletes a payment page in a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain : str
            Payment page identifier. The subdomain value is the last portion of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseGeneric2Part
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.paypoint.remove_page(
                entry="8cfec329267",
                subdomain="pay-your-fees-1",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.remove_page(entry, subdomain, request_options=request_options)
        return _response.data

    async def save_logo(
        self,
        entry: str,
        *,
        f_content: typing.Optional[str] = OMIT,
        filename: typing.Optional[str] = OMIT,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        furl: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Updates a paypoint logo.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        f_content : typing.Optional[str]
            Content of file, Base64-encoded. Ignored if furl is specified. Max upload size is 30 MB.

        filename : typing.Optional[str]
            The name of the attached file.

        ftype : typing.Optional[FileContentFtype]
            The MIME type of the file (if content is provided)

        furl : typing.Optional[str]
            Optional URL provided to show or download the file remotely

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.paypoint.save_logo(
                entry="8cfec329267",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.save_logo(
            entry, f_content=f_content, filename=filename, ftype=ftype, furl=furl, request_options=request_options
        )
        return _response.data

    async def settings_page(
        self, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SettingsQueryRecord:
        """
        Retrieves an paypoint's basic settings like custom fields, identifiers, and invoicing settings.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SettingsQueryRecord
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.paypoint.settings_page(
                entry="8cfec329267",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.settings_page(entry, request_options=request_options)
        return _response.data

    async def migrate(
        self,
        *,
        entry_point: Entrypointfield,
        new_parent_organization_id: int,
        notification_request: typing.Optional[NotificationRequest] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MigratePaypointResponse:
        """
        Migrates a paypoint to a new parent organization.

        Parameters
        ----------
        entry_point : Entrypointfield

        new_parent_organization_id : int
            The ID for the paypoint's new parent organization.

        notification_request : typing.Optional[NotificationRequest]
            Optional notification request object for a webhook

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MigratePaypointResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli
        from payabli.paypoint import NotificationRequest, WebHeaderParameter

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.paypoint.migrate(
                entry_point="473abc123def",
                new_parent_organization_id=123,
                notification_request=NotificationRequest(
                    notification_url="https://webhook-test.yoursie.com",
                    web_header_parameters=[
                        WebHeaderParameter(
                            key="testheader",
                            value="1234567890",
                        )
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.migrate(
            entry_point=entry_point,
            new_parent_organization_id=new_parent_organization_id,
            notification_request=notification_request,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .key_value import KeyValue


class SettingsQueryRecord(UniversalBaseModel):
    custom_fields: typing_extensions.Annotated[
        typing.Optional[typing.List[KeyValue]], FieldMetadata(alias="customFields")
    ] = pydantic.Field(default=None)
    """
    Any custom fields defined for the org.
    """

    for_invoices: typing_extensions.Annotated[
        typing.Optional[typing.List[KeyValue]], FieldMetadata(alias="forInvoices")
    ] = None
    for_pay_outs: typing_extensions.Annotated[
        typing.Optional[typing.List[KeyValue]], FieldMetadata(alias="forPayOuts")
    ] = None
    for_wallets: typing_extensions.Annotated[
        typing.Optional[typing.List[KeyValue]], FieldMetadata(alias="forWallets")
    ] = pydantic.Field(default=None)
    """
    Information about digital wallet settings for the entity. Available values are `isApplePayEnabled` and `isGooglePayEnabled`.
    """

    general: typing.Optional[typing.List[KeyValue]] = pydantic.Field(default=None)
    """
    
    """

    identifiers: typing.Optional[typing.List[KeyValue]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .button_element import ButtonElement
from .file_content import FileContent


class PageSetting(UniversalBaseModel):
    color: typing.Optional[str] = pydantic.Field(default=None)
    """
    An HTML color code in format #RRGGBB
    """

    custom_css_url: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="customCssUrl")] = (
        pydantic.Field(default=None)
    )
    """
    Complete URL to a custom CSS file to be loaded with the page
    """

    language: typing.Optional[str] = pydantic.Field(default=None)
    """
    Two-letter code following ISO 639-1
    """

    page_logo: typing_extensions.Annotated[typing.Optional[FileContent], FieldMetadata(alias="pageLogo")] = (
        pydantic.Field(default=None)
    )
    """
    Object containing logo file to upload/ use in page
    """

    payment_button: typing_extensions.Annotated[
        typing.Optional[ButtonElement], FieldMetadata(alias="paymentButton")
    ] = None
    redirect_after_approve: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="redirectAfterApprove")
    ] = pydantic.Field(default=None)
    """
    Flag indicating if the capability for redirection in the page will be activated
    """

    redirect_after_approve_url: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="redirectAfterApproveUrl")
    ] = pydantic.Field(default=None)
    """
    Complete URL where the page will be redirected after completion
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .created_at import CreatedAt
from .dbaname import Dbaname
from .entrypointfield import Entrypointfield
from .external_paypoint_id import ExternalPaypointId
from .legalname import Legalname
from .order_id import OrderId
from .org_parent_name import OrgParentName
from .page_identifier import PageIdentifier
from .query_payment_data import QueryPaymentData
from .query_transaction_payor_data import QueryTransactionPayorData
from .replyby import Replyby
from .transaction_query_records import TransactionQueryRecords
from .transaction_time import TransactionTime


class QueryChargebacksResponseRecordsItem(UniversalBaseModel):
    account_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="AccountType")] = (
        pydantic.Field(default=None)
    )
    """
    Type of account.
    """

    case_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="CaseNumber")] = pydantic.Field(
        default=None
    )
    """
    Case number of the chargeback.
    """

    chargeback_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="ChargebackDate")
    ] = pydantic.Field(default=None)
    """
    Date of the chargeback.
    """

    created_at: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="CreatedAt")] = None
    customer: typing_extensions.Annotated[
        typing.Optional[QueryTransactionPayorData], FieldMetadata(alias="Customer")
    ] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None
    id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="Id")] = pydantic.Field(default=None)
    """
    Unique identifier of the record.
    """

    last_four: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="LastFour")] = pydantic.Field(
        default=None
    )
    """
    Last four digits of the account number.
    """

    method: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Method")] = pydantic.Field(
        default=None
    )
    """
    Method of payment.
    """

    net_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="NetAmount")] = pydantic.Field(
        default=None
    )
    """
    Net amount after deductions.
    """

    order_id: typing_extensions.Annotated[typing.Optional[OrderId], FieldMetadata(alias="OrderId")] = None
    pageidentifier: typing.Optional[PageIdentifier] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = None
    payment_data: typing_extensions.Annotated[typing.Optional[QueryPaymentData], FieldMetadata(alias="PaymentData")] = (
        pydantic.Field(default=None)
    )
    """
    Payment data associated with the transaction.
    """

    payment_trans_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaymentTransId")] = (
        pydantic.Field(default=None)
    )
    """
    Transaction ID for the payment.
    """

    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname")] = (
        pydantic.Field(default=None)
    )
    """
    The 'Doing Business As' (DBA) name of the paypoint.
    """

    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[Entrypointfield], FieldMetadata(alias="PaypointEntryname")
    ] = pydantic.Field(default=None)
    """
    Entryname for the paypoint.
    """

    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname")
    ] = pydantic.Field(default=None)
    """
    Legal name of the paypoint.
    """

    reason: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Reason")] = pydantic.Field(
        default=None
    )
    """
    Description of the reason for chargeback.
    """

    reason_code: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ReasonCode")] = pydantic.Field(
        default=None
    )
    """
    Code representing the reason for chargeback.
    """

    reference_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ReferenceNumber")] = (
        pydantic.Field(default=None)
    )
    """
    Reference number for the transaction.
    """

    reply_by: typing_extensions.Annotated[typing.Optional[Replyby], FieldMetadata(alias="ReplyBy")] = None
    responses: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Responses")] = pydantic.Field(
        default=None
    )
    """
    Responses related to the transaction.
    """

    schedule_reference: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="ScheduleReference")] = (
        pydantic.Field(default=None)
    )
    """
    Reference for any scheduled transactions.
    """

    status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="Status")] = pydantic.Field(
        default=None
    )
    """
    Status of the transaction.
    """

    transaction: typing_extensions.Annotated[
        typing.Optional[TransactionQueryRecords], FieldMetadata(alias="Transaction")
    ] = None
    transaction_time: typing_extensions.Annotated[
        typing.Optional[TransactionTime], FieldMetadata(alias="TransactionTime")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .application_data_pay_in_services_ach import ApplicationDataPayInServicesAch
from .application_data_pay_in_services_card import ApplicationDataPayInServicesCard
from .odp_setup import OdpSetup


class ApplicationDataPayInServices(UniversalBaseModel):
    ach: ApplicationDataPayInServicesAch
    card: ApplicationDataPayInServicesCard
    odp: typing.Optional[OdpSetup] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
